@@ -4,14 +4,15 @@
  *
  *    Graphics Subsystem interface
  *
- *  Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005
- *     - The FreeType Development Team - www.freetype.org
+ *  Copyright (C) 1999-2025 by
+ *     - The FreeType Development Team - https://freetype.org
  *
  ***************************************************************************/
 
-#ifndef GRAPH_H
-#define GRAPH_H
+#ifndef GRAPH_H_
+#define GRAPH_H_
 
+#include "grtypes.h"
 #include "grevents.h"
 
  /*************************************************************************/
@@ -33,7 +34,7 @@
   /* pixel mode constants */
   typedef enum grPixelMode
   {
-    gr_pixel_mode_none = 0,
+    gr_pixel_mode_none = 0,    /* driver inquiry mode              */
     gr_pixel_mode_mono,        /* monochrome bitmaps               */
     gr_pixel_mode_pal4,        /* 4-bit paletted - 16 colors       */
     gr_pixel_mode_pal8,        /* 8-bit paletted - 256 colors      */
@@ -46,6 +47,7 @@
     gr_pixel_mode_lcdv,        /* vertical RGB-decimated           */
     gr_pixel_mode_lcd2,        /* horizontal BGR-decimated         */
     gr_pixel_mode_lcdv2,       /* vertical BGR-decimated           */
+    gr_pixel_mode_bgra,        /* premultiplied BGRA colors        */
 
     gr_pixel_mode_max          /* don't remove */
 
@@ -94,7 +96,6 @@
   } grBitmap;
 
 
-
   typedef long   grPos;
   typedef char   grBool;
 
@@ -106,14 +107,54 @@
   } grVector;
 
 
+ /*********************************************************************
+  *
+  * <Union>
+  *   grColor
+  *
+  * <Description>
+  *   a generic color pixel with arbitrary depth up to 32 bits.
+  *
+  * <Fields>
+  *   value  :: for colors represented using bitfields
+  *   chroma :: for colors in a particular byte order
+  *
+  * <Note>
+  *   Use grFindColor to set grColor corresponding to grBitmap.
+  *
+  ********************************************************************/
+
   typedef union grColor_
   {
-    long           value;
+    uint32_t       value;
     unsigned char  chroma[4];
 
   } grColor;
 
 
+ /**********************************************************************
+  *
+  * <Function>
+  *    grFindColor
+  *
+  * <Description>
+  *    return grColor pixel appropriate for a target mode.
+  *
+  * <Input>
+  *    target  :: used to match the pixel mode
+  *    red     :: red value in 0..255 range
+  *    green   :: green value in 0..255 range
+  *    blue    :: blue value in 0..255 range
+  *    alpha   :: alpha value in 0..255 range
+  *
+  ********************************************************************/
+  extern grColor
+  grFindColor( grBitmap*  target,
+               int        red,
+               int        green,
+               int        blue,
+               int        alpha );
+
 
  /**********************************************************************
   *
@@ -121,7 +162,8 @@
   *    grNewBitmap
   *
   * <Description>
-  *    creates a new bitmap
+  *    Creates a new bitmap or resizes an existing one.  The allocated
+  *    pixel buffer is not initialized.
   *
   * <Input>
   *    pixel_mode   :: the target surface's pixel_mode
@@ -136,8 +178,10 @@
   *    Error code. 0 means success.
   *
   * <Note>
-  *    This function really allocates a pixel buffer, zero it, then
-  *    returns a descriptor for it.
+  *    This function really allocates a pixel buffer, then returns
+  *    a descriptor for it.
+  *
+  *    An existing bitmap will be resized.
   *
   *    Call grDoneBitmap when you're done with it..
   *
@@ -150,101 +194,22 @@
                             grBitmap    *bit );
 
 
- /**********************************************************************
-  *
-  * <Function>
-  *    grBlitGlyphToBitmap
-  *
-  * <Description>
-  *    writes a given glyph bitmap to a target surface.
-  *
-  * <Input>
-  *    target  :: handle to target bitmap
-  *    glyph   :: handle to source glyph bitmap
-  *    x       :: position of left-most pixel of glyph image in target surface
-  *    y       :: position of top-most pixel of glyph image in target surface
-  *    color   :: color to be used to draw a monochrome glyph
-  *
-  * <Return>
-  *   Error code. 0 means success
-  *
-  * <Note>
-  *   There are only two supported source pixel modes : monochrome
-  *   and gray. The 8-bit images can have any number of grays between
-  *   2 and 128, and conversions to the target surface is handled
-  *   _automatically_.
-  *
-  *   Note however that you should avoid blitting a gray glyph to a gray
-  *   bitmap with fewer levels of grays, as this would much probably
-  *   give unpleasant results..
-  *
-  *   This function performs clipping
-  *
-  **********************************************************************/
-
-  extern int
-  grBlitGlyphToBitmap( grBitmap*  target,
-                       grBitmap*  glyph,
-                       grPos      x,
-                       grPos      y,
-                       grColor    color );
-
-
-  /* values must be in 0..255 range */
-  extern grColor
-  grFindColor( grBitmap*  target,
-               int        red,
-               int        green,
-               int        blue,
-               int        alpha );
-
-#if 0
-
- /**********************************************************************
-  *
-  * <Function>
-  *    grFillRectangle
-  *
-  * <Description>
-  *    this function is used to fill a given rectangle on a surface
-  *
-  * <Input>
-  *    surface :: handle to target surface
-  *    x       :: x coordinate of the top-left corner of the rectangle
-  *    y       :: y coordinate of the top-left corner of the rectangle
-  *    width   :: rectangle width in pixels
-  *    height  :: rectangle height in pixels
-  *    color   :: fill color
-  *
-  **********************************************************************/
-
-  extern void  grFillRectangle( grBitmap*  surface,
-                                grPos      x,
-                                grPos      y,
-                                grPos      width,
-                                grPos      height,
-                                grColor    color );
-
-#endif /* 0 */
-
-
  /**********************************************************************
   *
   * <Function>
   *    grWriteCellChar
   *
   * <Description>
-  *    The graphics sub-system contains an internal Latin1 8x8 font
-  *    which can be used to display simple strings of text without
-  *    using FreeType.
+  *    The graphics sub-system contains an internal CP437 font which can
+  *    be used to display simple strings of text without using FreeType.
   *
-  *    This function writes a single 8x8 character on the target bitmap.
+  *    This function writes a single character on the target bitmap.
   *
   * <Input>
   *    target   :: handle to target surface
   *    x        :: x pixel position of character cell's top left corner
   *    y        :: y pixel position of character cell's top left corner
-  *    charcode :: Latin-1 character code
+  *    charcode :: CP437 character code
   *    color    :: color to be used to draw the character
   *
   **********************************************************************/
@@ -263,17 +228,16 @@
   *    grWriteCellString
   *
   * <Description>
-  *    The graphics sub-system contains an internal Latin1 8x8 font
-  *    which can be used to display simple strings of text without
-  *    using FreeType.
+  *    The graphics sub-system contains an internal CP437 font which can
+  *    be used to display simple strings of text without using FreeType.
   *
-  *    This function writes a string with the internal font
+  *    This function writes a string with the internal font.
   *
   * <Input>
   *    target       :: handle to target bitmap
   *    x            :: x pixel position of string's top left corner
   *    y            :: y pixel position of string's top left corner
-  *    string       :: Latin-1 text string
+  *    string       :: CP437 text string
   *    color        :: color to be used to draw the character
   *
   **********************************************************************/
@@ -285,6 +249,7 @@
                            const char* string,
                            grColor     color );
 
+
  /**********************************************************************
   *
   * <Function>
@@ -305,6 +270,24 @@
   extern  void  grDoneBitmap( grBitmap*  bit );
 
 
+ /**********************************************************************
+  *
+  * <Function>
+  *    grFillRect
+  *
+  * <Description>
+  *    this function is used to fill a given rectangle on a surface
+  *
+  * <Input>
+  *    target  :: handle to target surface
+  *    x       :: x coordinate of the top-left corner of the rectangle
+  *    y       :: y coordinate of the top-left corner of the rectangle
+  *    width   :: rectangle width in pixels
+  *    height  :: rectangle height in pixels
+  *    color   :: fill color
+  *
+  **********************************************************************/
+
   extern void
   grFillHLine( grBitmap*  target,
                int        x,
@@ -356,7 +339,7 @@
   *    "device chain"
   *
   * <Fields>
-  *    name   :: ASCII name of the device, e.g. "x11", "os2pm", etc..
+  *    name   :: ASCII name of the device, e.g. "x11", "win32", etc..
   *    device :: handle to the device descriptor.
   *    next   :: next element in chain
   *
@@ -445,17 +428,9 @@
   *    error code. 0 means success. invalid device name otherwise
   *
   * <Note>
-  *    All drivers are _required_ to support at least the following
-  *    pixel formats :
-  *
-  *    - gr_pixel_mode_mono : i.e. monochrome bitmaps
-  *    - gr_pixel_mode_gray : with any number of gray levels between
-  *                           2 and 256.
-  *
-  *    the pixel modes do not provide the number of grays in the case
-  *    of "gray" devices. You should try to create a surface with the
-  *    maximal number (256, that is) and see the value returned in
-  *    the bitmap descriptor.
+  *    This feature is not implemented in the common drivers.  Use
+  *    grNewSurface with gr_pixel_mode.none instead to let the driver
+  *    set the mode.
   *
   **********************************************************************/
 
@@ -464,7 +439,6 @@
                                  grPixelMode*  *pixel_modes );
 
 
-
  /**********************************************************************
   *
   * <Function>
@@ -485,21 +459,15 @@
   *               for the surface. the bitmap's 'pitch' and 'buffer'
   *               fields are ignored on input.
   *
-  *               On output, the bitmap describes the surface's image
-  *               completely. It is possible to write directly in it
-  *               with grBlitGlyphToBitmap, even though the use of
-  *               grBlitGlyphToSurface is recommended.
-  *
   * <Return>
   *    handle to the corresponding surface object. 0 in case of error
   *
   * <Note>
-  *    All drivers are _required_ to support at least the following
-  *    pixel formats :
+  *    If the requsted mode is gr_pixel_mode_none, the driver chooses
+  *    a mode that is convenient for the device.
   *
-  *    - gr_pixel_mode_mono : i.e. monochrome bitmaps
-  *    - gr_pixel_mode_gray : with any number of gray levels between
-  *                           2 and 256.
+  *    All drivers are _required_ to support at least the following
+  *    pixel formats: gray, rgb555, rgb565, rgb24, or rgb32.
   *
   *    This function might change the bitmap descriptor's fields. For
   *    example, when displaying a full-screen surface, the bitmap's
@@ -517,6 +485,7 @@
 
   extern void  grDoneSurface( grSurface*  surface );
 
+
  /**********************************************************************
   *
   * <Function>
@@ -560,6 +529,42 @@
   extern void  grRefreshSurface( grSurface*  surface );
 
 
+ /**********************************************************************
+  *
+  * <Function>
+  *    grBlitGlyphToSurface
+  *
+  * <Description>
+  *    writes a given glyph bitmap to a target surface.
+  *
+  * <Input>
+  *    surface :: handle to surface
+  *    glyph   :: handle to source glyph bitmap
+  *    x       :: position of left-most pixel of glyph image in target surface
+  *    y       :: position of top-most pixel of glyph image in target surface
+  *    color   :: color to be used to draw a monochrome glyph
+  *
+  * <Return>
+  *   Error code. 0 means success
+  *
+  * <Note>
+  *   The function performs gamma-corrected alpha blending using the source
+  *   bitmap as an alpha channel(s), which can be specified for each pixel
+  *   as 8-bit (gray) bitmaps, or for individual color channels in various
+  *   LCD arrangements.
+  *
+  *   This function performs clipping.  It also handles mono and BGRA
+  *   bitmaps without gamma correction.
+  *
+  **********************************************************************/
+
+  extern int
+  grBlitGlyphToSurface( grSurface*  surface,
+                        grBitmap*   glyph,
+                        grPos       x,
+                        grPos       y,
+                        grColor     color );
+
 
  /**********************************************************************
   *
@@ -570,17 +575,16 @@
   *    This function is equivalent to calling grWriteCellChar on the
   *    surface's bitmap, then invoking grRefreshRectangle.
   *
-  *    The graphics sub-system contains an internal Latin1 8x8 font
-  *    which can be used to display simple strings of text without
-  *    using FreeType.
+  *    The graphics sub-system contains an internal CP437 font which can
+  *    be used to display simple strings of text without using FreeType.
   *
-  *    This function writes a single 8x8 character on the target bitmap.
+  *    This function writes a single character on the target bitmap.
   *
   * <Input>
   *    target   :: handle to target surface
   *    x        :: x pixel position of character cell's top left corner
   *    y        :: y pixel position of character cell's top left corner
-  *    charcode :: Latin-1 character code
+  *    charcode :: CP437 character code
   *    color    :: color to be used to draw the character
   *
   **********************************************************************/
@@ -602,17 +606,16 @@
   *    This function is equivalent to calling grWriteCellString on the
   *    surface's bitmap, then invoking grRefreshRectangle.
   *
-  *    The graphics sub-system contains an internal Latin1 8x8 font
-  *    which can be used to display simple strings of text without
-  *    using FreeType.
+  *    The graphics sub-system contains an internal CP437 font which can
+  *    be used to display simple strings of text without using FreeType.
   *
-  *    This function writes a string with the internal font
+  *    This function writes a string with the internal font.
   *
   * <Input>
   *    target       :: handle to target bitmap
   *    x            :: x pixel position of string's top left corner
   *    y            :: y pixel position of string's top left corner
-  *    string       :: Latin-1 text string
+  *    string       :: CP437 text string
   *    color        :: color to be used to draw the character
   *
   **********************************************************************/
@@ -643,6 +646,28 @@
                            const char* title_string );
 
 
+ /**********************************************************************
+  *
+  * <Function>
+  *    grSetIcon
+  *
+  * <Description>
+  *    set the icon of a given windowed surface.
+  *
+  * <Input>
+  *    surface :: handle to target surface
+  *    icon    :: handle to icon bitmap
+  *
+  * <Return>
+  *    the next appropriate icon size in pixels.
+  *
+  * <Note>
+  *    Returns the largest appropriate icon size if icon is NULL.
+  *
+  **********************************************************************/
+
+  extern int  grSetIcon( grSurface*  surface,
+                         grBitmap*   icon );
 
 
  /**********************************************************************
@@ -661,7 +686,7 @@
   *    event  :: the returned event
   *
   * <Note>
-  *    XXX : For now, only keypresses are supported.
+  *    Only keypresses and resizing events are supported.
   *
   **********************************************************************/
 
@@ -673,20 +698,44 @@
  /**********************************************************************
   *
   * <Function>
-  *    grSetGlyphGamma
+  *    grSetTargetGamma
   *
   * <Description>
   *    set the gamma-correction coefficient. This is only used to
   *    blit glyphs
   *
   * <Input>
+  *    surface    :: handle to target surface
   *    gamma      :: gamma value. <= 0 to select sRGB transfer function
   *
   **********************************************************************/
 
   extern
-  void  grSetGlyphGamma( double  gamma_value );
+  void  grSetTargetGamma( grSurface*  surface,
+                          double      gamma_value );
+
+
+ /**********************************************************************
+  *
+  * <Function>
+  *    grSetTargetPenBrush
+  *
+  * <Description>
+  *    set the pen position and brush color as required for direct mode.
+  *
+  * <Input>
+  *    surface    :: handle to target surface
+  *    x, y       :: pen position
+  *    color      :: color as defined by grFindColor
+  *
+  **********************************************************************/
+
+  extern
+  void  grSetTargetPenBrush( grSurface*  surface,
+                             int         x,
+                             int         y,
+                             grColor     color );
 
 /* */
 
-#endif /* GRAPH_H */
+#endif /* GRAPH_H_ */
